@@ -1,10 +1,11 @@
 package Tk::ProgressBar;
 
 use vars qw($VERSION);
-$VERSION = '4.006'; # $Id: //depot/Tkutf8/Tk/ProgressBar.pm#6 $
+$VERSION = '4.012'; # was: sprintf '4.%03d', q$Revision: #10 $ =~ /\D(\d+)\s*$/;
 
 use Tk;
 use Tk::Canvas;
+use Tk::Trace;
 use Carp;
 use strict;
 
@@ -86,7 +87,7 @@ sub _arrange {
     my $from = $c->{Configure}{'-from'};
     my $to   = $c->{Configure}{'-to'};
     my $horz = $c->{Configure}{'-anchor'} =~ /[ew]/i ? 1 : 0;
-    my $dir  = $c->{Configure}{'-anchor'} =~ /[ne]/i ? -1 : 1;
+    my $dir  = $c->{Configure}{'-anchor'} =~ /[se]/i ? -1 : 1;
 
     my($minv,$maxv) = $from < $to ? ($from,$to) : ($to,$from);
 
@@ -131,7 +132,7 @@ sub _arrange {
 	my $colors = $c->{Configure}{'-colors'} || [];
 	my $bdir = $from < $to ? $dir : 0 - $dir;
 
-	$c->delete($c->find('all'));
+	$c->delete('all');
 
 	$c->createRectangle(0,0,$w+$x*2,$h+$y*2,
 		-fill =>  $c->{Configure}{'-troughcolor'},
@@ -296,27 +297,27 @@ sub value {
 
 sub variable {
     my $c = shift;
-    my $val = \$c->{'-variable'};
-    my $old = $$val;
+    my $oldvarref = $c->{'-variable'};
+    my $oldval = $$oldvarref if $oldvarref;
     if(@_) {
-	my $value = shift;
-        if (ref $old)
+	my $varref = shift;
+        if ($oldvarref)
          {
-          $c->{'-value'} = $$old;
-          untie $$old if tied($$old);
+	  $c->traceVdelete($oldvarref);
          }
-        tie $$value,'Tk::Configure',$c,'-value';
-	$$val = $value;
+	$c->{'-variable'} = $varref;
+	$c->traceVariable($varref, 'w', sub { $c->value($_[1]) });
+	$$varref = $oldval;
 	_layoutRequest($c,2);
     }
-    $old;
+    $oldval;
 }
 
 sub Destroyed
 {
  my $c = shift;
  my $var = delete $c->{'-variable'};
- untie $$var if (defined($var) && ref($var))
+ $c->traceVdelete($var);
 }
 
 1;
@@ -449,7 +450,7 @@ full progress bar will be displayed. Defaults to 100.
 =item B<-variable>
 
 Specifies the reference to a scalar variable to link to the ProgressBar.
-Whenever the value of the variable changes, the ProgressBar will upate
+Whenever the value of the variable changes, the ProgressBar will update
 to reflect this value. (See also the B<value> method below.)
 
 =item B<-value>
