@@ -1,4 +1,4 @@
-# Copyright (c) 1995-2000 Nick Ing-Simmons.
+# Copyright (c) 1995-2004 Nick Ing-Simmons.
 # Copyright (c) 1999 Greg London.
 # All rights reserved.
 # This program is free software; you can redistribute it and/or
@@ -6,7 +6,7 @@
 package Tk::TextUndo;
 
 use vars qw($VERSION $DoDebug);
-$VERSION = '4.009'; # $Id: //depot/Tkutf8/Tk/TextUndo.pm#9 $
+$VERSION = '4.014'; # $Id: //depot/Tkutf8/Tk/TextUndo.pm#15 $
 $DoDebug = 0;
 
 use Tk qw (Ev);
@@ -571,6 +571,7 @@ sub Insert
  $w->addGlobStart;
  $w->SUPER::Insert($char);
  $w->addGlobEnd;
+ $w->see('insert');
 }
 
 
@@ -589,6 +590,7 @@ sub InsertKeypress
     {
      $w->SUPER::insert($index,$char);
      $undo_item->[2] = $w->index('insert');
+     $w->see('insert');
      return;
     }
   }
@@ -652,6 +654,13 @@ sub FileName
  return $w->{'FILENAME'};
 }
 
+sub PerlIO_layers
+{
+ my ($w,$layers) = @_;
+ $w->{PERLIO_LAYERS} = $layers if @_ > 1;
+ return $w->{PERLIO_LAYERS} || '' ;
+}
+
 sub ConfirmDiscard
 {
  my ($w)=@_;
@@ -702,19 +711,19 @@ sub Save
  my ($w,$filename) = @_;
  $filename = $w->FileName unless defined $filename;
  return $w->FileSaveAsPopup unless defined $filename;
- local *FILE;
- if (open(FILE,">$filename"))
+ my $layers = $w->PerlIO_layers;
+ if (open(my $file,">$layers",$filename))
   {
    my $status;
    my $count=0;
    my $index = '1.0';
    my $progress;
-   my ($lines) = $w->index('end') =~ /^(\d+)\./;
+   my ($lines) = $w->index('end - 1 chars') =~ /^(\d+)\./;
    while ($w->compare($index,'<','end'))
     {
 #    my $end = $w->index("$index + 1024 chars");
      my $end = $w->index("$index  lineend +1c");
-     print FILE $w->get($index,$end);
+     print $file $w->get($index,$end);
      $index = $end;
      if (($count++%1000) == 0)
       {
@@ -722,7 +731,7 @@ sub Save
       }
     }
    $progress->withdraw if defined $progress;
-   if (close(FILE))
+   if (close($file))
     {
      $w->ResetUndo;
      $w->FileName($filename);
@@ -741,22 +750,23 @@ sub Load
  my ($w,$filename) = @_;
  $filename = $w->FileName unless (defined($filename));
  return 0 unless defined $filename;
- local *FILE;
- if (open(FILE,"<$filename"))
+ my $layers = $w->PerlIO_layers;
+ if (open(my $file,"<$layers",$filename))
   {
    $w->MainWindow->Busy;
    $w->EmptyDocument;
    my $count=1;
    my $progress;
-   while (<FILE>)
+   while (<$file>)
     {
      $w->SUPER::insert('end',$_);
      if (($count++%1000) == 0)
       {
-       $progress = $w->TextUndoFileProgress (Loading => $filename,$count,tell(FILE),-s $filename);
+       $progress = $w->TextUndoFileProgress (Loading => $filename,
+                         $count,tell($file),-s $filename);
       }
     }
-   close(FILE);
+   close($file);
    $progress->withdraw if defined $progress;
    $w->markSet('insert' => '1.0');
    $w->FileName($filename);
@@ -773,23 +783,25 @@ sub IncludeFile
  my ($w,$filename) = @_;
  unless (defined($filename))
   {$w->BackTrace("filename not specified"); return;}
- if (open(FILE,"<$filename"))
+ my $layers = $w->PerlIO_layers;
+ if (open(my $file,"<$layers",$filename))
   {
    $w->Busy;
    my $count=1;
    $w->addGlobStart;
    my $progress;
-   while (<FILE>)
+   while (<$file>)
     {
      $w->insert('insert',$_);
      if (($count++%1000) == 0)
       {
-       $progress = $w->TextUndoFileProgress(Including => $filename,$count,tell(FILE),-s $filename);
+       $progress = $w->TextUndoFileProgress(Including => $filename,
+                        $count,tell($file),-s $filename);
       }
     }
    $progress->withdraw if defined $progress;
    $w->addGlobEnd;
-   close(FILE);
+   close($file);
    $w->Unbusy;
   }
  else
@@ -910,7 +922,7 @@ sub MarkSelectionsSavePositions
 sub RestoreSelectionsMarkedSaved
 {
  my ($w)=@_;
- my $i = 0;
+ my $i = 1;
  my %mark_hash;
  foreach my $mark ($w->markNames)
   {
@@ -932,10 +944,30 @@ sub RestoreSelectionsMarkedSaved
 
 ####################################################################
 # selected lines may be discontinous sequence.
-sub SelectedLineNumbers
+sub GetMarkedSelectedLineNumbers
 {
  my ($w) = @_;
- my @ranges = $w->tagRanges('sel');
+
+ my $i = 1;
+ my %mark_hash;
+ my @ranges;
+ foreach my $mark ($w->markNames)
+  {
+   $mark_hash{$mark}=1;
+  }
+
+ while(1)
+  {
+   my $markstart = 'MarkSelectionsSavePositions_'.$i++;
+   last unless(exists($mark_hash{$markstart}));
+   my $indexstart = $w->index($markstart);
+   my $markend = 'MarkSelectionsSavePositions_'.$i++;
+   last unless(exists($mark_hash{$markend}));
+   my $indexend = $w->index($markend);
+
+   push(@ranges, $indexstart, $indexend);
+  }
+
  my @selection_list;
  while (@ranges)
   {
@@ -959,7 +991,7 @@ sub insertStringAtStartOfSelectedLines
  my ($w,$insert_string)=@_;
  $w->addGlobStart;
  $w->MarkSelectionsSavePositions;
- foreach my $line ($w->SelectedLineNumbers)
+ foreach my $line ($w->GetMarkedSelectedLineNumbers)
   {
    $w->insert($line.'.0', $insert_string);
   }
@@ -973,7 +1005,7 @@ sub deleteStringAtStartOfSelectedLines
  $w->addGlobStart;
  $w->MarkSelectionsSavePositions;
  my $length = length($insert_string);
- foreach my $line ($w->SelectedLineNumbers)
+ foreach my $line ($w->GetMarkedSelectedLineNumbers)
   {
    my $start = $line.'.0';
    my $end   = $line.'.'.$length;
